#   Copyright 2020-2022 Exactpro (Exactpro Systems Limited)
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

import logging
from typing import Any, Dict


logger = logging.getLogger(__name__)


class AbstractConfiguration:

    def check_unexpected_args(self, kwargs: Dict[str, Any]) -> None:
        if len(kwargs) > 0:
            logger.warning(f'{self.__class__.__name__} JSON config contains unexpected arguments: {kwargs}')
