#   Copyright 2021-2022 Exactpro (Exactpro Systems Limited)
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

from prometheus_client import Gauge

from th2_common.schema.metrics.abstract_metric import AbstractMetric


class PrometheusMetric(AbstractMetric):

    def __init__(self, name: str, documentation: str) -> None:
        self.metric = Gauge(name=name, documentation=documentation)

        super().__init__()

    def on_value_change(self, value: bool) -> None:
        self.metric.set(1.0 if value else 0.0)
